@@ -1,8 +1,8 @@
 # Copyright © 2017 Tom Hacohen
 #
 # This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, version 3.
+# it under the terms of the GNU Lesser General Public License as
+# published by the Free Software Foundation, version 3.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -58,19 +58,28 @@ class EteSync:
         if directory != '' and not os.path.exists(directory):
             os.makedirs(directory)
 
-        database = SqliteExtDatabase(db_path, pragmas={'foreign_keys': 1})
+        database = SqliteExtDatabase(db_path, pragmas={
+            'journal_mode': 'wal',
+            'foreign_keys': 1,
+            })
         database.connect()
 
         self._set_db(database)
 
     def _init_db_tables(self, database, additional_tables=None):
+        CURRENT_DB_VERSION = 2
+
+        new_db = not database.table_exists('journalentity')
+
         database.create_tables([cache.Config, pim.Content, cache.User, cache.JournalEntity,
                                 cache.EntryEntity, cache.UserInfo], safe=True)
         if additional_tables:
             database.create_tables(additional_tables, safe=True)
 
-        db_version = cache.Config.get_or_none()
-        if db_version is None:
+        default_db_version = CURRENT_DB_VERSION if new_db else 0
+        config, created = cache.Config.get_or_create(defaults={'db_version': default_db_version})
+
+        if config.db_version < 1:
             from playhouse.migrate import SqliteMigrator, migrate
             # Essentially version 0 so do first migration.
             migrator = SqliteMigrator(database)
@@ -83,7 +92,20 @@ class EteSync:
                 # A hack because we don't have a db config yet.
                 pass
 
-            cache.Config.insert(db_version=1).execute()
+            config.db_version = 1
+            config.save()
+
+
+        if config.db_version < 2:
+            from playhouse.migrate import SqliteMigrator, migrate
+            migrator = SqliteMigrator(database)
+
+            migrate(
+                migrator.add_column('journalentity', 'remote_last_uid', cache.JournalEntity.remote_last_uid),
+            )
+
+            config.db_version = 2
+            config.save()
 
     def get_or_create_user_info(self, force_fetch=False):
         user_info = None
@@ -142,6 +164,7 @@ class EteSync:
             journal.version = entry.version
             journal.encrypted_key = entry.encrypted_key
             journal.read_only = entry.read_only
+            journal.remote_last_uid = entry.remote_last_uid
             journal.content = entry.getContent()
             journal.save()
 
@@ -212,6 +235,9 @@ class EteSync:
         prev = self._get_last_entry(journal)
         last_uid = None if prev is None else prev.uid
 
+        if (last_uid is not None) and (last_uid == journal.remote_last_uid):
+            return
+
         for entry in manager.list(crypto_manager, last_uid):
             entry.verify(prev)
             content = entry.getContent()
